<?php 

/**
 * 
 * Reservations API 
 *
 * This file provides custom API endpoints for the booking desks plugin. It includes an authentication endpoint to
 * generate JWT tokens based on provided credentials, and a revoke endpoint to invalidate JWT tokens, essentially
 * logging out users on the client-side. The plugin uses the firebase/php-jwt library for JWT token encoding and decoding.
 * 
 */


// Get all reservations endpoint
function booking_desks_get_all_reservations_endpoint()
{
    // Get the JWT token from the request headers
    $headers = getallheaders();
    if (!validate_api_key($headers)) {
        return rest_ensure_response(array('error' => 'Invalid API key.'), 401);
    }

    if (!isset($headers['Authorization'])) {
        return rest_ensure_response(array('error' => 'Missing Authorization header.'), 400);
    }

    $token = $headers['Authorization'];
    $secret_key = YOUR_SECRET_KEY;

    // Validate the JWT token
    $validation_result = validate_jwt_token($token, $secret_key);
    if (!$validation_result['valid']) {
        return rest_ensure_response(array('error' => 'Invalid token or signature.'), 401);
    }

    global $wpdb;
    $currentTimestamp = current_time('mysql');

    $query = "SELECT *
          FROM {$wpdb->prefix}client_info ci
          JOIN {$wpdb->prefix}booking_info bi ON ci.client_id = bi.client_id
          WHERE bi.date_end > '{$currentTimestamp}'";
    $results =  $wpdb->get_results($query);
    console_log($results);

    if (!empty($results)) {
        $output = array();
        foreach ($results as $row) {
            $desk = get_post($row->desk_id);
            $building = get_the_terms($row->desk_id, 'buildings');
            $location = get_the_terms($row->desk_id, 'location');
            $row_data = array(
                'client_id' => $row->client_id,
                'first_name' => $row->first_name,
                'last_name' => $row->last_name,
                'booking_id' => $row->booking_id,
                'desk_id' => $row->desk_id,
                'pass_code' => $row->pass_code,
                'room' => $row->room,
                'date_start' => $row->date_start,
                'date_end' => $row->date_end,
            );

            $output[] = $row_data;
        }
        return rest_ensure_response($output, 200);
    } else {
        return rest_ensure_response(array(), 200);
    }
}


add_action('rest_api_init', function () {
    register_rest_route('booking-desks/v1', '/reservations', array(
        'methods' => 'GET',
        'callback' => 'booking_desks_get_all_reservations_endpoint',
    ));
});


// Check if pass code is valid endpoint
function booking_desks_check_pass_code_endpoint()
{
    // Get the JWT token from the request headers
    $headers = getallheaders();
    if (!validate_api_key($headers)) {
        return rest_ensure_response(array('error' => 'Invalid API key.'), 401);
    }

    if (!isset($headers['Authorization'])) {
        return rest_ensure_response(array('error' => 'Missing Authorization header.'), 400);
    }

    $token = $headers['Authorization'];
    $secret_key = YOUR_SECRET_KEY;

    // Validate the JWT token
    $validation_result = validate_jwt_token($token, $secret_key);
    if (!$validation_result['valid']) {
        return rest_ensure_response(array('error' => $validation_result['error']), 401);
    }

    // Get the pass code and room ID from the query parameters
    $pass_code = isset($_GET['pass_code']) ? absint($_GET['pass_code']) : 0;
    $room_id = isset($_GET['room_id']) ? absint($_GET['room_id']) : 0;


    if (empty($pass_code) || empty($room_id)) {
        return rest_ensure_response(array('error' => 'Invalid request. Please provide both pass code and room ID.'), 400);
    }

    global $wpdb;
    $current_time = current_time('mysql');
    $query = $wpdb->prepare(
        "SELECT * FROM {$wpdb->prefix}booking_info 
        WHERE pass_code = %d AND room = %d AND date_start <= %s AND date_end >= %s",
        $pass_code,
        $room_id,
        $current_time,
        $current_time
    );

    $result = $wpdb->get_row($query);

    if (!empty($result)) {
        return rest_ensure_response(array('valid' => true), 200);
    } else {
        return rest_ensure_response(array('valid' => false), 200);
    }
}

add_action('rest_api_init', function () {
    register_rest_route('booking-desks/v1', '/check-pass-code', array(
        'methods' => 'GET',
        'callback' => 'booking_desks_check_pass_code_endpoint',
    ));
});